
#ifdef _WINDOWS
	#include <windows.h>
#endif

#include "lcodialog.h"

#include <qapplication.h>
#include <qclipboard.h>
#include <qcheckbox.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qlistbox.h>
#include <qpushbutton.h>
#include <qradiobutton.h>
#include <qspinbox.h>
#include <qtoolbutton.h>
#include <qlayout.h>
#include <qvariant.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qpainter.h>
#include <qimage.h>
#include <qpixmap.h>
#include <qmenubar.h>
#include <qpopupmenu.h>
#include <qfiledialog.h>
#include <qmessagebox.h>
#include <qcolor.h>


#include "up_image.xpm"
#include "down_image.xpm"
#include "pin_c.xpm"
#include "pin_n.xpm"
#include "pin_s.xpm"
#include "pin_nc.xpm"
#include "pin_norm.xpm"
#include "pin_nul.xpm"

#include <string.h>
#include <assert.h>
#include <list>
#include <string>
using namespace std;

#include "coord.h"
#include "ce3io.h"
#include "complib.h"
#include "compeditwnd.h"

#include "compattrbimpl.h"
#include "inputaliasimpl.h"
#include "lcoicon.xpm"
#include "lcosmallicon.xpm"



void SLCoListItem::paint (QPainter * p)
{
	QFont font;
	QFontInfo fontInfo(font);
	QColor col;
	int pixelSize = fontInfo.pixelSize();
	//p->drawText(0,pixelSize,text());
	const char* pname = m_pCompIndex->name();
	if(m_bAlias){
		col = QColor("red");
		p->setPen(col);
	}
//	else{
//		col = QColor("black");
//	}

	//p->drawText(0,15,QString::fromLocal8Bit(pname));
	p->drawText(0,pixelSize,QString::fromLocal8Bit(pname));
}

int SLCoListItem::height ( const QListBox * ) const
{
	QFont font;
	QFontInfo fontInfo(font);
	int pixelSize =fontInfo.pixelSize();
	return pixelSize+2;
}

int SLCoListItem::width ( const QListBox * ) const
{
	return 120;
}

//void SLCoListItem::setItemText( const QString& text)
//{
//	setText(text);
//}


/* 
 *  Constructs a lcoDialog which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 *
 *  The dialog will by default be modeless, unless you set 'modal' to
 *  TRUE to construct a modal dialog.
 */
lcoDialog::lcoDialog( QWidget* parent,  const char* name, bool modal, WFlags fl )
    : QDialog( parent, name, modal, fl )
{

	setIcon(QPixmap((const char**)lcosmallicon ));

	menuBar = new QMenuBar(this);
	QPopupMenu * file = new QPopupMenu( this );
    menuBar->insertItem( "&File", file );
 	file->insertItem( "&New", this, SLOT(fileNew()), CTRL+Key_N  );
 	file->insertItem( "&Open...", this, SLOT(fileOpen()),  CTRL+Key_O );
 	file->insertItem( "&Save", this, SLOT(fileSave()),  CTRL+Key_S );
 	file->insertItem( "Save &as...", this, SLOT(fileSaveAs()) );
	file->insertSeparator();
 	file->insertItem( "E&xit", this, SLOT(fileExit()) );

	QPopupMenu * component = new QPopupMenu( this );
    menuBar->insertItem( "&Component", component );
 	component->insertItem( "Cu&t", this, SLOT(componentCut()), CTRL+Key_X  );
 	component->insertItem( "&Copy", this, SLOT(componentCopy()),  CTRL+Key_C );
 	component->insertItem( "&Paste", this, SLOT(componentPaste()),  CTRL+Key_V );
 	component->insertItem( "&Delete", this, SLOT(componentDel()) );
	component->insertSeparator();
 	component->insertItem( "&New...", this, SLOT(componentNew()) );
 	component->insertItem( "Att&ribute...", this, SLOT(componentAttribute()) );
 	component->insertItem( "&Alias...", this, SLOT(componentAlias()) );


	//Pin
	QPopupMenu * pin = new QPopupMenu( this );
    menuBar->insertItem( "&Pin", pin );
	pin->insertItem( "&Insert", this, SLOT(pinInsert()), CTRL+Key_I  );
 	pin->insertItem( "&Delete", this, SLOT(pinDelete()), CTRL+Key_D  );

	//Help
	QPopupMenu * help = new QPopupMenu( this );
    menuBar->insertItem( "&Help", help );
	help->insertItem( "&About", this, SLOT(helpAbout()));
	help->insertItem( "About&Qt", this, SLOT(aboutQt()));


	//ġܥΤΥ᡼ν
    QPixmap image_up( ( const char** ) up_image );
    QPixmap image_down( ( const char** ) down_image );
    //QPixmap image_pin_n( ( const char** ) pin_n );
    //QPixmap image_pin_s( ( const char** ) pin_s );
    //QPixmap image_pin_nc( ( const char** ) pin_nc );
    //QPixmap image_pin_c( ( const char** ) pin_c );
    //QPixmap image_pin_nul( ( const char** ) pin_nul );
    //QPixmap image_pin_norm( ( const char** ) pin_norm );


    if ( !name ) setName( "lcoDialog" );
    resize( 463, 362 );
    setProperty( "caption", tr( "Qt-LCo" ) );
    setProperty( "sizeGripEnabled", QVariant( FALSE, 0 ) );
    lcoDialogLayout = new QGridLayout( this );
    lcoDialogLayout->setSpacing( 6 );
    lcoDialogLayout->setMargin( 4 );
	lcoDialogLayout->setMenuBar(menuBar);




    Layout3 = new QVBoxLayout;
    Layout3->setSpacing( 6 );
    Layout3->setMargin( 0 );

    Layout2 = new QHBoxLayout;
    Layout2->setSpacing( 6 );
    Layout2->setMargin( 0 );

    button_up = new QToolButton( this, "button_up" );
    button_up->setProperty( "text", tr( "" ) );
    button_up->setProperty( "pixmap", image_up );
    Layout2->addWidget( button_up );

    button_down = new QToolButton( this, "button_down" );
    button_down->setProperty( "sizePolicy", QSizePolicy( (QSizePolicy::SizeType)5, (QSizePolicy::SizeType)0, button_down->sizePolicy().hasHeightForWidth() ) );
    button_down->setProperty( "text", tr( "" ) );
    button_down->setProperty( "pixmap", image_down );
    Layout2->addWidget( button_down );
    Layout3->addLayout( Layout2 );

    ListBox1 = new QListBox( this, "ListBox1" );
    //ListBox1->insertItem( tr( "New Item" ) );
    ListBox1->setProperty( "sizePolicy", QSizePolicy( (QSizePolicy::SizeType)0, (QSizePolicy::SizeType)7, ListBox1->sizePolicy().hasHeightForWidth() ) );
    ListBox1->setProperty( "minimumSize", QSize( 140, 300 ) );
    ListBox1->setProperty( "vScrollBarMode", (int)QListBox::AlwaysOn );
    Layout3->addWidget( ListBox1 );

    lcoDialogLayout->addMultiCellLayout( Layout3, 0, 1, 0, 0 );

	CompEdit = new SCompEditWnd(this, "CompEdit");
	lcoDialogLayout->addMultiCellWidget(CompEdit,0,1,1,1);

	//QSpacerItem* spacer = new QSpacerItem( 20, 20, QSizePolicy::Minimum, QSizePolicy::Expanding );
    //lcoDialogLayout->addItem( spacer, 1, 1 );

    // signals and slots connections
	connect(ListBox1	,SIGNAL( selectionChanged ()	),this,SLOT( listboxSelectionChanged()	));
	connect(button_up	,SIGNAL( clicked ()				),this,SLOT( onButtonUp()				));
	connect(button_down	,SIGNAL( clicked ()				),this,SLOT( onButtonDown()				));
	connect(CompEdit	,SIGNAL( modifiedSignal()		),this,SLOT( onCompEditModified()		));
    //connect( buttonOk, SIGNAL( clicked() ), this, SLOT( accept() ) );
    //connect( buttonCancel, SIGNAL( clicked() ), this, SLOT( reject() ) );

	m_compLib=new SCompLib;
	m_bDirty=false;
	m_strDocumentPath="";
}

/*  
 *  Destroys the object and frees any allocated resources
 */
lcoDialog::~lcoDialog()
{
	delete m_compLib;
}





void lcoDialog::accept()
{
	//qDebug("lcoDialog::accept()");
	QDialog::accept();
}

void lcoDialog::reject()
{
	//qDebug("lcoDialog::reject()");
}

void lcoDialog::closeEvent( QCloseEvent* ce )
{
	//qDebug("lcoDialog::closeEvent()");
	if(!fileNew()){
		ce->ignore();
	}else{
		ce->accept();
	}
}

void lcoDialog::onCompEditModified()
{
	m_bDirty=true;
}


void lcoDialog::initDocument()
{
	ListBox1->clear();
	CompEdit->setCompIndex(NULL);
	delete m_compLib;
	m_compLib=new SCompLib;
	m_bDirty=false;
	m_strDocumentPath="";
}


bool lcoDialog::fileNew()
{
	if(m_bDirty){
		switch( QMessageBox::information( this, "Qt-LCo",
				      "The current document has been changed since the last save.\nDo you want to save it?",
				      "Save", "No", "Cancel",
				      0, 2) ) {
		case 0:
			if(!fileSave()) return false;
			break;
		case 1:
			break;
		case 2:
			return false;
		}
	}
	initDocument();
	setDialogCaption();
	return true;
}

void lcoDialog::fileOpen()
{
	if(m_bDirty){
		switch( QMessageBox::information( this, "Qt-LCo",
				      "The current document has been changed since the last save.\nDo you want to save it?",
				      "Save", "No", "Cancel",
				      0, 2) ) {
		case 0:
			if(!fileSave()) return;
			break;
		case 1:
			break;
		case 2:
			return;
		}
	}
	QString fn = QFileDialog::getOpenFileName( QString::null,"*.lb3",this );
	if ( !fn.isEmpty() ) {
		initDocument();
		readLibrary(fn);
	}
	return;
}

bool lcoDialog::fileSave()
{
	if(m_strDocumentPath.isEmpty()){
		return fileSaveAs();
	}else{
		return writeLibrary(m_strDocumentPath);
	}
}

bool lcoDialog::fileSaveAs()
{
	QString fn = QFileDialog::getSaveFileName( QString::null,"*.lb3",this );
	if ( !fn.isEmpty() ) {
		//ĥҤ̵϶Ū.ce3Ȥ롣
		QFileInfo finfo(fn);
		if(finfo.extension(false).isEmpty()){	//xxxx.פȡxxxxפξĥҤʤȽꤷƤޤ
			if(fn.right(1)!=".") fn+=".";		// Ǹʸ.פǤʤС.פɲá
			fn+="lb3";							// ĥҤɲá
		}
		if(QFile::exists (fn)){
			QString msg=fn;
			msg += " already exists.Overwrite?";
			if( QMessageBox::information( this, "Save as",msg,
				      "Overwrite", "Cancel",QString::null,0, 1) ==0) {
				return writeLibrary(fn);
			}
			return false;		
  		}else{
			return writeLibrary(fn);
		}
	}
	return false;
}


void lcoDialog::setDialogCaption()
{

	QString caption("Qt-LCo ");
//	if(!m_strDocumentPath.isEmpty()){
		QFileInfo finfo(m_strDocumentPath);
		caption += finfo.fileName();
//	}

#ifdef _WINDOWS
	HWND hw = winId();
	char szCaption[300];
	strcpy(szCaption,caption.local8Bit());
	::SetWindowText(hw,szCaption);
	topData()->caption = caption;
#else
	setCaption( caption );
#endif
}



void lcoDialog::fileExit()
{

	close();
}

void lcoDialog::componentCut()
{
    componentCopy();
	componentDel();
}

void lcoDialog::componentCopy()
{
	int nSelected = ListBox1->currentItem();
	if(nSelected < 0) return;
	SLCoListItem* itemCurrent = (SLCoListItem*)ListBox1->item(nSelected);

	string name = itemCurrent->m_pCompIndex->name();
	string ref  = itemCurrent->m_pCompIndex->compInfo()->m_pszRef;


	string str;
	SWriteCE3 wce3;
	wce3.Attach(&str);

	wce3.WriteRecord("+BSCH3_DATA_V.1.0");	wce3.WriteEOL();
	wce3.WriteRecord("+COMPONENT");			wce3.WriteEOL();

	wce3.WriteRecord("+BSCH3_LIB_V.1.0");	wce3.WriteEOL();

	itemCurrent->m_pCompIndex->compInfo()->writeCe3(wce3,itemCurrent->m_pCompIndex->name());

	wce3.WriteRecord("-BSCH3_LIB_V.1.0");	wce3.WriteEOL();
	wce3.WriteRecordInt("X",itemCurrent->m_pCompIndex->compInfo()->m_size.w()*10);
	wce3.WriteRecordInt("Y",itemCurrent->m_pCompIndex->compInfo()->m_size.h()*10);
	wce3.WriteRecordString("LIB",name);
	wce3.WriteRecordString("N",  name);
	wce3.WriteRecordString("R",  ref);
	wce3.WriteRecord("-COMPONENT");			wce3.WriteEOL();
	wce3.WriteRecord("-BSCH3_DATA_V.1.0");	wce3.WriteEOL();

    QClipboard *cb = QApplication::clipboard();
    cb->setText(QString::fromLocal8Bit(str.c_str()));

}

void lcoDialog::componentPaste()
{
	QString strID("+BSCH3_LIB_V.1.0");
	QClipboard *cb = QApplication::clipboard();
    QString strTop = cb->text().left(strID.length());
	//qDebug("*%s*   *%s*",strID.latin1(),strTop.latin1());
	//if(strID != strTop) return;
	if(strID != strTop){
		strID=QString("+BSCH3_DATA_V.1.0");
		strTop = cb->text().left(strID.length());
		if(strID != strTop) return;
	}

	string cbbuff=string(cb->text().local8Bit());
	SReadCE3 rce3;
	rce3.Attach(cbbuff.c_str());

	if(rce3.SkipTo("+BSCH3_LIB_V.1.0")==EOF) return;

//	string str;
//	rce3.ReadRecord(str);
//	if(strcmp(str.c_str(),strID.local8Bit())) return;

	int nCountOld = m_compLib->count();
	
	if(m_compLib->ReadLb3(rce3)){
		int nSelected = ListBox1->currentItem();
		int nCountNew = m_compLib->count() - nCountOld;
		SCompIndexListIterator iteIndex = m_compLib->m_listCompIndex.end();
		while(nCountNew>0){
			iteIndex--;
			nCountNew--;
			SLCoListItem* listItem = new SLCoListItem;
			listItem->m_pCompIndex = *iteIndex;
			ListBox1->insertItem(listItem,nSelected);
		}
		if(nSelected<0) nSelected=0;
		ListBox1->setCurrentItem(nSelected);
		m_bDirty = true;
	}
	m_bDirty=true;


	//readCe3(rce3,&m_listTemp,&pt,NULL);
	//if(!m_listTemp.size()) return false;

}

void lcoDialog::componentDel()
{
	int nSelected = ListBox1->currentItem();
	if(nSelected<0) return;

	SLCoListItem* itemSelected = (SLCoListItem*)ListBox1->item(nSelected);
	
	int nCount = ListBox1->count();
	int n;
	bool existAlias=false;
	for(n=0;n<nCount;n++){
		SLCoListItem* listItem = (SLCoListItem*)ListBox1->item(n);
		if( (n != nSelected) && (listItem->m_pCompIndex->compInfo() == itemSelected->m_pCompIndex->compInfo())){
			existAlias=true;
			break;
		}
	}
	//ꥢ¸ߤʤϡΤ
	if(!existAlias){
		SCompInfo* compinfo = itemSelected->m_pCompIndex->compInfo();
		SCompInfoListIterator iteInfo = m_compLib->m_listCompInfo.begin();
		while(iteInfo != m_compLib->m_listCompInfo.end()){
			if(*iteInfo == compinfo){
				delete (*iteInfo);
				m_compLib->m_listCompInfo.erase(iteInfo);
				break;
			}
			iteInfo++;
		}
	}
	//ǥåκ
	SCompIndex* compIndex = itemSelected->m_pCompIndex;
	SCompIndexListIterator iteIndex = m_compLib->m_listCompIndex.begin();
	while(iteIndex != m_compLib->m_listCompIndex.end()){
		if(*iteIndex == compIndex){
			delete (*iteIndex);
			m_compLib->m_listCompIndex.erase(iteIndex);
			break;
		}
		iteIndex++;
	}
	//ꥹȥܥåκ
	ListBox1->removeItem(nSelected);
	if(ListBox1->count()>0){
		nSelected = ListBox1->currentItem();
		listboxSelectionChanged();
		//ListBox1->setCurrentItem(nSelected-1);
		//ListBox1->setCurrentItem(nSelected);
		itemSelected = (SLCoListItem*)ListBox1->item(nSelected);
		CompEdit->setCompIndex(itemSelected->m_pCompIndex);
	}else{
		CompEdit->setCompIndex(NULL);
	}
	m_bDirty=true;

}

void lcoDialog::componentNew()
{
	compattrb *dlg = new compattrb(this,0,true);
	if(dlg->exec()){
		//̾η
		string name = string(dlg->name().latin1());
		name = m_compLib->uniqueName(name);

		//ʾ
		SCompInfo* pComp = new SCompInfo;
		pComp->setBlock(dlg->block());
		pComp->setSize(SSize(dlg->width(),dlg->height()),dlg->usePattern());
		pComp->setRef(dlg->ref().latin1());

		//ʥǥå
		SCompIndex* pIndex = new SCompIndex;
		pIndex->setName(name.c_str());
		pIndex->setCompInfo(pComp);

		//ʾȥǥåꥹȤɲ
		m_compLib->m_listCompIndex.push_back(pIndex);
		m_compLib->m_listCompInfo.push_back(pComp);

		//ꥹȥܥåɲ
		int nSelected = ListBox1->currentItem();
		SLCoListItem* listItem = new SLCoListItem;
		listItem->m_pCompIndex = pIndex;
		ListBox1->insertItem(listItem,nSelected);

		m_bDirty=true;
	}
	delete dlg;
}

void lcoDialog::componentAttribute()
{
	int nSelected = ListBox1->currentItem();
	if(nSelected<0) return;
	compattrb *dlg = new compattrb(this,0,true);
	SLCoListItem* itemSelected = (SLCoListItem*)ListBox1->item(nSelected);
	SCompIndex* pCompIndex = itemSelected->m_pCompIndex;
	SCompInfo* pComp = pCompIndex->m_pCompInfo;
	dlg->setName(QString::fromLatin1(pCompIndex->name()));
	dlg->setRef(QString::fromLatin1(pComp->m_pszRef));
	dlg->setWidth(pComp->m_size.w());
	dlg->setHeight(pComp->m_size.h());
	dlg->setBlock(pComp->m_nBlock);
	dlg->setUsePattern(pComp->m_pPtn!=NULL);
	
	if(dlg->exec()){
		bool change=false;
		//̾η
		string name = string(dlg->name().latin1());
		if(pCompIndex->name() != name){
			name = m_compLib->uniqueName(name); //̾ѹäȤˡ̾롣
			pCompIndex->setName(name.c_str());
			change=true;
		}
		
		//ֹѹ
		string ref = string(dlg->ref().latin1());
		if(pComp->m_pszRef != ref){
			pComp->setRef(ref.c_str());
			change=true;
		}

		//֥åѹ
		if(pComp->m_nBlock != dlg->block()){
			pComp->setBlock(dlg->block());
			change=true;
		}

		//ѥѢԻѤѹäν
		//ѹ˾õʤȥСإåɤˤʤ롣
		if(pComp->m_pPtn!=NULL && !dlg->usePattern()){
			pComp->setUsePattern(false);
			change=true;
		}

		//ѹɬפʾˤϥѥ
		SSize size(dlg->width(),dlg->height());
		if(size != pComp->m_size || (pComp->m_pPtn==NULL && dlg->usePattern())){
			pComp->setSize(size,dlg->usePattern());
			change=true;
		}

		if(change){
			ListBox1->viewport()->update();
			CompEdit->setCompIndex(pCompIndex);
			m_bDirty=true;
		}
	}
	delete dlg;
}

void lcoDialog::componentAlias()
{
	int nSelected = ListBox1->currentItem();
	if(nSelected<0) return;
	inputalias *dlg = new inputalias(this,0,true);
	SLCoListItem* itemSelected = (SLCoListItem*)ListBox1->item(nSelected);
	SCompIndex* pCompIndex = itemSelected->m_pCompIndex;
	SCompInfo* pComp = pCompIndex->m_pCompInfo;
	dlg->setName(QString::fromLatin1(pCompIndex->name()));
	if(dlg->exec() && !dlg->alias().isEmpty()){
		SCompIndex* pAliasCompIndex = new SCompIndex;
		string name = string(dlg->alias().latin1());
		name = m_compLib->uniqueName(name); //ˡ̾롣
		pAliasCompIndex->setName(name.c_str());
		pAliasCompIndex->setCompInfo(pComp);
		m_compLib->m_listCompIndex.push_back(pAliasCompIndex);
		SLCoListItem* listItem = new SLCoListItem;
		listItem->m_pCompIndex = pAliasCompIndex;
		ListBox1->insertItem(listItem,nSelected+1);
		ListBox1->setCurrentItem(nSelected+1);
		m_bDirty=true;
	}
	delete dlg;
}

void lcoDialog::pinInsert()
{
    CompEdit->pinInsert();
}

void lcoDialog::pinDelete()
{
	CompEdit->pinDelete();
}

void lcoDialog::aboutQt()
{
	QMessageBox::aboutQt( this, "About Qt" );
}



void lcoDialog::helpAbout()
{
	QPixmap icon( (const char**)lcoicon );

	QMessageBox mb(this);

	mb.setCaption("About  Qt-LCo");
	mb.setTextFormat(RichText);
	mb.setText(	"<center><b>Qt-LCo  Version 0.05</b><br><br>"
					"Copyright(C) 2002-2004 H.Okada(Suigyodo)<br>"
					"Qt-LCo comes with absolutely no warranty.<br>"
					"URL  http://www.suigyodo.com<br>"
					"E-Mail  hitoshi@suigyodo.com</center>"
		);
	mb.setIconPixmap(icon);
	//mb.adjustSize();
	mb.exec();
}

bool lcoDialog::writeLibrary(const QString& filename)
{

	SWriteCE3 wce3;
	//string str;
	//SCompIndexListIterator iteIndex;

	FILE* fp=fopen(filename.local8Bit(),"w");
	if(fp==NULL) return false;
	wce3.Attach(fp);
	wce3.WriteRecord("+BSCH3_LIB_V.1.0");
	wce3.WriteEOL();

	int nCount = ListBox1->count();
	int nIndex;

	for(nIndex=0; nIndex<nCount; nIndex++){
		SLCoListItem* itemCurrent = (SLCoListItem*)ListBox1->item(nIndex);
		int n;
		SLCoListItem* listItem=NULL;
		for(n=0;n<nIndex;n++){
			listItem = (SLCoListItem*)ListBox1->item(n);
			if(listItem->m_pCompIndex->compInfo() == itemCurrent->m_pCompIndex->compInfo()) break;
		}
		if(n<nIndex){
			SCompLib::WriteAlias(wce3,itemCurrent->m_pCompIndex->name(),listItem->m_pCompIndex->name());
			wce3.WriteEOL();
		}else{
			itemCurrent->m_pCompIndex->compInfo()->writeCe3(wce3,itemCurrent->m_pCompIndex->name());
			wce3.WriteEOL();
		}
	}

	wce3.WriteRecord("-BSCH3_LIB_V.1.0");
	wce3.WriteEOL();
	fclose(fp);
	m_strDocumentPath = filename;
	setDialogCaption();
	m_bDirty=false;
	return true;
}
bool lcoDialog::readLibrary(const QString& filename)
{
	SRect rc;
	if(m_compLib->readLibraryFile(filename.local8Bit())){
		//qDebug("readLibrary()  component count = %d",m_compLib->count());
		m_strDocumentPath = filename;
		m_bDirty=false;
		setListBox();
		setDialogCaption();
		return true;
	}
	m_bDirty=false;
	return false;
}

void lcoDialog::setListBox()
{
	ListBox1->clear();

	SCompIndexListIterator iteIndex = m_compLib->m_listCompIndex.begin();
	while(iteIndex != m_compLib->m_listCompIndex.end()){
//		const char* pname=(*iteIndex)->name();
		SLCoListItem* listItem = new SLCoListItem;
//		listItem->setItemText(QString::fromLocal8Bit(pname));
		listItem->m_pCompIndex = *iteIndex;
//		ListBox1->insertItem(QString::fromLocal8Bit(pname));
		ListBox1->insertItem(listItem);
		iteIndex++;
	}

}

void lcoDialog::listboxSelectionChanged()
{
	int nSelected = ListBox1->currentItem();
	//qDebug("lcoDialog::listboxSelectionChanged()  %d",nSelected);
	SLCoListItem* itemSelected = (SLCoListItem*)ListBox1->item(nSelected);
	int nCount = ListBox1->count();
	int n;
	for(n=0;n<nCount;n++){
		SLCoListItem* listItem = (SLCoListItem*)ListBox1->item(n);
		if(n == nSelected){
			listItem->m_bAlias = false;
		}else{
			listItem->m_bAlias = (listItem->m_pCompIndex->compInfo() == itemSelected->m_pCompIndex->compInfo());
		}
	}
	ListBox1->viewport()->update();
	if(CompEdit->modified()) m_bDirty=true;
	if(itemSelected){
		CompEdit->setCompIndex(itemSelected->m_pCompIndex);
	}else{
		CompEdit->setCompIndex(NULL);
	}
}

void lcoDialog::onButtonUp()
{
	int nSelected = ListBox1->currentItem();
	if(nSelected < 1) return;
	SLCoListItem* itemSelected = (SLCoListItem*)ListBox1->item(nSelected);
	SLCoListItem* listItem = new SLCoListItem;
	listItem->m_pCompIndex = itemSelected->m_pCompIndex;
	ListBox1->insertItem(listItem,nSelected-1);
	ListBox1->setCurrentItem(nSelected-1);
	ListBox1->removeItem(nSelected+1);
	m_bDirty=true;
}

void lcoDialog::onButtonDown()
{
	int nSelected = ListBox1->currentItem();
	if(nSelected < 0) return;
	int nCount = ListBox1->count();
	if(nSelected > nCount -2) return;

	SLCoListItem* itemSelected = (SLCoListItem*)ListBox1->item(nSelected);
	SLCoListItem* listItem = new SLCoListItem;
	listItem->m_pCompIndex = itemSelected->m_pCompIndex;
	ListBox1->insertItem(listItem,nSelected+2);
	ListBox1->setCurrentItem(nSelected+2);
	ListBox1->removeItem(nSelected);
	m_bDirty=true;
}
